/*
 * Copyright (C) 2023 Mattia Chiozzi <mattia@chiozzi.cc>
 * License: BSD0
 */

/*
 * bat - battery-friendly battery indicator
 *
 * bat is used to efficiently display the remaining battery in tmux.
 * The output consists of the 1st char of the battery percentage, and if no
 * battery is found it shows an '-'. Then, it sends a xnotify(1) notification
 * if battery < LOW_BATTERY (default: 20%)
 */

#include <fcntl.h>
#include <machine/apmvar.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define LOW_BATTERY 20
#define NOTIFY_CMD "x 'Low battery! (20%)'"

/* file used to prevent notifications */
#define F "/tmp/bat_prevent_notify"

static int
load_apm_power_info(struct apm_power_info *apm_info)
{
	int fd;

	if ((fd = open("/dev/apm", O_RDONLY)) < 0)
		return 0;

	if (ioctl(fd, APM_IOC_GETPOWER, apm_info) < 0) {
		close(fd);
		return 0;
	}

	return close(fd), 1;
}

int
main(void)
{
	struct apm_power_info apm_info;
	int low;

	if (load_apm_power_info(&apm_info)) {
		if (apm_info.battery_state == APM_BATTERY_ABSENT)
			printf("%c", '-');
		else {
			printf("%d", apm_info.battery_life/10);
			if (apm_info.battery_life == LOW_BATTERY) {
				if (apm_info.ac_state != APM_AC_ON) {
					if (access(F, F_OK) < 0) {
						low = open(F, O_CREAT, 0664);
						close(low);
						system(NOTIFY_CMD);
					}
				}
			} else if (!access(F, F_OK))
				remove(F);
		}
	}

	return 0;
}
